#       This file is part of SurrealServices.
#
#       SurrealServices is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       SurrealServices is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with SurrealServices; if not, write to the Free Software
#       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package securitybot::ss2tkl;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT = qw(update_ss_timed update_ss) }

use SrSv::Shared qw( $last_version );

########################################################################
#                       SecureServ definitions                         #
########################################################################
my $ss_base_url = 'http://secure.irc-chat.net/defs.php';

sub get_ss_definitions(%) {
	my (%conf) = @_;
	my $url = "'$ss_base_url?u=".$conf{'ss_Username'}.'&p='.$conf{'ss_Pass'}."'";

	my %skipfilters;
	foreach my $skipfilter (split(/ /, $conf{'ss_SkipFilters'})) {
		$skipfilters{uc($skipfilter)} = 1;
	}
	
	open ((my $DATAFILE), '-|', ('wget -q -O - ' . $url));
	#open ((my $DATAFILE), "$main::prefix/data/viri.dat");

	my (@filters, $version);
	while (my $l = <$DATAFILE>) {
	#XcoderBot 2 0 0 "^Xcoder.*" "You appear to be infected with a BotNet Trojan" 1
		chomp $l;
		if($l =~ /^(\S+) (\d) (\d) (\d) \"(.+)\" \"(.+)\" (\d)$/) {
			my ($filter, $type, $regex, $reason, $action) = ($1,$2,$5,$6,$7);
			next if $skipfilters{uc($filter)};
			my $ref = make_ss_tkl($filter, $type, $regex, $reason, $action);
			push @filters, $ref if $ref;
		}
		if ($l =~ /^(\d+)$/) {
			$version = $1;
		}
	}
	close $DATAFILE;
	return ($version, @filters);
}

sub make_ss_tkl($$$$$) {
	my ($filter, $type, $regex, $reason, $action) = @_;
	my ($tkl_target, $tkl_action);
	$regex =~ s/^\s+//;
	$reason =~ s/ /_/g;
	if($regex =~ /\(\?\i\)/) {
		$regex =~ s/\(\?\i\)//;
	}
	else {
		$regex = "(?i)$regex";
	}
	if($type == 0) {
		if ($regex =~ /^\^/ or $regex =~ /\$$/) {
			$regex =~ s/^\^/\^\x01/;
			$regex =~ s/\$$/\x01\$/;
		}
		else {
			$regex = "\x01$regex\x01";
		}
		$tkl_target = 'n';
	}
	elsif($type == 1 or $type == 6) {
		$tkl_target = 'cpnNPq';
	}
	elsif($type == 2) {
		$tkl_target = 'u';
		$regex =~ s/\$$//;
		$regex = "$regex!";
	}
	elsif($type == 3) {
		$tkl_target = 'u';
		$regex =~ s/\$$//;
		$regex =~ s/^\^//;
		$regex = "!$regex@";
	}
	elsif($type == 4) {
		$tkl_target = 'u';
		$regex =~ s/^\^//;
		$regex = ":$regex";
	}
	elsif($type == 5) {
		# We don't have a spamfilter type for this
		# this would be a drone channel.
		return undef;
	}

	if($action == 0) {
		$tkl_action = 'v'; # viruschan
	}
	elsif($action == 1) {
		$tkl_action = 'Z'; # gzline. gline would be better emulation
	}
	elsif($action == 2 or $action == 3) {
		$tkl_action = 'b'; # block.
		# would be better to trap but not block, but there is no such spamfilter action.
	}

	return [$tkl_target, $tkl_action, "$filter:_$reason", $regex];
}

########################################################################
#   END                 SecureServ definitions                   END   #
########################################################################

sub update_ss_timed($%) {
	my ($time, %conf) = @_;

	add_timer('3600', $time, __PACKAGE__, "securitybot::update_ss_timed");

	update_ss(%conf);
}

sub update_ss(%) {
	my (%conf) = @_;
	my %ss_filters;
	diagmsg( " -- Loading SecureServ definitions.");
	$securitybot::get_all_spamfilter->execute();
	while(my ($target, $action, $setter, $expire, $time, $bantime, $reason, $regex) =
		$securitybot::get_all_spamfilter->fetchrow_array())
	{
		next unless $setter eq $securitybot::sbnick;
		$securitybot::ss_filters{"$target $action $reason $regex"} = [$target, $action, $reason, $regex];
	}
	$securitybot::get_all_spamfilter->finish();

	my ($version, @filters) = get_ss_definitions(%conf);
	if ($last_version and $version <= $last_version) {
		diagmsg( " -- SecureServ definitions already up-to-date.");
		return;
	}
	$last_version = $version;
	unless (@filters) {
		diagmsg( " -- Loading SecureServ definitions failed.");
		return;
	}

	foreach my $tmp (@filters) {
		my ($target, $action, $reason, $regex) = @$tmp;
		if(!defined($ss_filters{"$target $action $reason $regex"})) {
			ircd::spamfilter(+1, $target, $action, $securitybot::sbnick, $conf{'DroneKlineTime'}, $reason, $regex);
		}
		delete($ss_filters{"$target $action $reason $regex"});
	}

	foreach my $filter_str (keys(%ss_filters)) {
		my ($target, $action, $reason, $regex) = @{$ss_filters{$filter_str}};
		ircd::spamfilter(0, $target, $action, $securitybot::sbnick, $conf{'DroneKlineTime'}, $reason, $regex);
	}
	diagmsg( " -- Loading SecureServ definitions succeeded.");
	
}

1;
